.Dd Jan 24, 2008
.Dt drood 8
.Sh NAME
.Nm Drood
.Nd Minimal KQueue-Based HTTP/1.1 Server
.Sh SYNOPSIS
.Nm drood -r Ao server-root Ac Oo -m Ao instances Ac -i Ao interface Ac -p Ao port Ac -e Ao root-resource Ac -d Ao cgi-dir Ac -t Ao read-timeout Ac -w Ao write-timeout Ac -u Ao user Ac -g Ao group Ac -q Ao backlog Ac -h Ao SCGI host Ac -s Ao SCGI port Ac -c -x -n Oc
.Sh DESCRIPTION
Drood is a lightweight, kqueue-based, connection-multiplexing, HTTP/1.1
server, intended as an alternative to more complex web servers for websites
where the server requirements are simple.  Drood supports name-based
virtual hosting, CGI and SCGI, and will service both IPv4 and IPv6
connections.
.Pp
Drood reads a client request, and spits out the requested resource, or an
error message, back at the client.  Drood does not perform content
negotiation, nor access authentication, nor does it provide any response
headers useful for caching server responses beyond the "Date" and
"Last-Modified" headers.  The server recognizes the HEAD, GET, and POST
request methods.
.Pp
By default, FreeBSD limits each process to having a maximum of 64
simultaneously open descriptors.  This places a limit on the number of
connections drood may multiplex.  The sysctl settings below, allow for a
generous number of connections.  These values can be set at the
command-line with the "sysctl" utility.  It is recommended that they be
enabled permanently by placing these lines in /etc/sysctl.conf on the host
machine.
.Bd -literal -offset left
kern.maxfiles=16384
kern.maxfilesperproc=16384
kern.kq_calloutmax=16384
kern.ipc.maxsockets=16384
.Ed
.Pp
Drood writes its pid into /var/run/drood.pid, if it can (ie., it is started
as root), and may be stopped with a SIGTERM.  A rc.d script is provided and
installed in /usr/local/etc/rc.d/.  Add the following lines to
/etc/rc.conf to start drood on system boot-up.  Replace the items in
brackets with values appropriate for your system, and/or add or remove
options as suits your system.  The available options are described in full
at the end of this manual page.
.Bd -literal -offset left
drood_enable="YES"
drood_flags="-r <server-root> -u <user> -g <group>"
.Ed
.Pp
If your hardware has multiple cores and/or processors, and you want
maximize drood's throughput, then add the -m option to run multiple
instances of drood, one for each core.
.Pp
If transaction logging is not crucial to your setup, then consider adding
the -n option.  It suppresses transaction logging and consequently will
allow the server to service requests at a higher rate.
.Pp
Once your /etc/rc.conf is configured you may start, stop, or restart drood,
or determine if it is running with the following commands:  
.Bd -literal -offset left
/usr/local/etc/rc.d/drood start
/usr/local/etc/rc.d/drood stop
/usr/local/etc/rc.d/drood restart
/usr/local/etc/rc.d/drood status
.Ed
.Pp
If you do not want drood started on system start, then set
.Bd -literal -offset left
drood_enable="NO"
.Ed
.Pp
and use the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/drood forcestart
/usr/local/etc/rc.d/drood forcestop
/usr/local/etc/rc.d/drood forcerestart
/usr/local/etc/rc.d/drood forcestatus
.Ed
.Pp
These are the mappings from filename suffixes to MIME types which drood
recognizes out-of-the-box.  They are listed in types.tab in
/usr/local/etc/drood/.  Any file which cannot be identified by these
mappings is identified by the server as "text/plain; charset=utf-8".  To
add a mapping, simply add a line to types.tab.  Each line must consist of
the filename suffix to be mapped, followed by one or more tab characters,
followed by the MIME type, followed by a newline.  Blank lines are ignored,
after the first line.  The first line must not be blank.  If drood finds
empty fields in types.tab it will refuse to run.  If you modify types.tab,
you must restart drood for the changes to take effect.
.Bl -column -offset left "jpeg" "application/x-shockwave-flash"
.It Sy Suffix Ta Sy MIME Type
.It html Ta text/html; charset=utf-8
.It htm  Ta text/html; charset=utf-8
.It xml  Ta text/xml; charset-utf-8
.It txt  Ta text/plain; charset=utf-8
.It css  Ta text/css
.It ico  Ta image/vnd.microsoft.icon
.It png  Ta image/png
.It jpg  Ta image/jpeg
.It jpeg Ta image/jpeg
.It tiff Ta image/tiff
.It tif  Ta image/tiff
.It gif  Ta image/gif
.It tgz  Ta application/x-compressed
.It zip  Ta application/zip
.It ps   Ta application/postscript
.It eps  Ta application/postscript
.It gz   Ta application/x-gzip
.It tgz  Ta application/x-compressed
.It pdf  Ta application/pdf
.It swf  Ta application/x-shockwave-flash
.It js   Ta application/x-javascript
.It mov  Ta video/quicktime
.It qt   Ta video/quicktime
.It mpg  Ta video/mpeg
.It mp2  Ta video/mpeg
.It mpa  Ta video/mpeg
.It mpe  Ta video/mpeg
.It avi  Ta video/x-msvideo
.It asf  Ta video/x-ms-asf
.It asx  Ta video/x-ms-asf
.It asr  Ta video/x-ms-asf
.It flv  Ta video/x-flv
.It ra   Ta audio/x-pn-realaudio
.It ram  Ta audio/x-pn-realaudio
.It wav  Ta audio/x-wav
.It mp3  Ta audio/mpeg
.El
.Pp
The following options are recognized.  Only the -r option is required, but
you will probably want to include at least the -u, and -g options when you
are starting the server for production use.  For optimal performance, the
-q and -t options should be tweaked to match your load levels.  If you wish
to run CGI and/or SCGI programs, add the -d option.  If you wish to run
SCGI programs, add one or both of the -h and -s options.
.Bl -tag -width "-r"
.It -r
The -r option is mandatory and specifies the server root directory, which
drood will make its current working directory.
.Pp
The resources served-up by drood are determined by the "Host" header of
requests, thus implementing name-based virtual hosting.  For every hostname
you wish to be recognized by the server, an identically-named subdirectory
of the server root directory must be created.  The static files served up
by each virtual host are placed into the identically-named subdirectory.
Any or all of the subdirectories may be symbolic links.  One subdirectory
must be named "default".  This is the directory which will be used to
fulfill static requests when no "Host" header is supplied by a client (eg.,
HTTP/1.0 requests).  The "default" directory can and should be a symbolic
link.  If you would like a directory to be used for more than one hostname,
you must create links for the extra hostnames.
.Pp
For example, I could have, in my server root directory, the following
subdirectories.  Note the symlink for the loopback address.  Without it,
drood will not be able to find resources for requests from loopback
connections.  Note the link for the IP address of the server, to allow
clients to connect using URIs containing the server's IP address instead of
its hostname.  Note that if you are using IPv6, you will need to create
links which include the surrounding square brackets which are used to
identify IPv6 addresses in URIs (rfc3986).
.Bd -literal -offset left
mammothcheese.ca     -> actual subdirectory
127.0.0.1            -> symlink to mammothcheese.ca
[::1]                -> symlink to mammothcheese.ca (IPv6 Loopback)
99.250.61.49         -> symlink to mammothcheese.ca
default              -> symlink to mammothcheese.ca
www.mammothcheese.ca -> symlink to mammothcheese.ca
.Ed
.Pp
Drood will reject any request whose host header contains a virgule "/",
or starts with "." in order to prevent clients from accessing resources
outside of the virtual host subdirectories.  Note this does not prevent
you from symlinking a directory outside of the server root directory
hierarchy into the server root directory, if the server is not running
chrooted (the -c option).  Take care what you expose to the world, my sons
and daughters.
.Pp
Remember to set the ownership and permissions of the directories and their
contents according to the values you give for the -u and -g options.
Static resources must be set readable by the user or group drood will be
running as.  Directories must similarly be searchable.
.Pp
If the -d option is present, requests whose "Host" headers reference the
directory specified by that option, will be rejected.  This prevents
clients from requesting data files in the (S)CGI directory.  Drood will
refuse to send files to which it has executable permission to clients
regardless of where they reside.
.Pp
Directory names are read once, at startup.  If you change the directory
structure in the server root directory, then you must restart drood to have
the new structure recognized.  Drood does a linear search through its list
of directories, so it is not suited to truly massive virtual hosting, which
a service provider might employ.
.It -e
The root resource of all virtual hosts defaults to a static resource named
"index.html", which itself may be a symbolic link.  This may be changed with
the -e option.  Note that this changes the name of the root resource of all
virtual hosts served-up by drood.  You may, with the -e option, specify a
dynamic resource as the root resource.  If you do, you must be sure to
include the -d option to tell the server where (S)CGI programs live.  Eg.,
.Bd -literal -offset left
drood -r /usr/local/www/drood -d cgi-bin -e /cgi-bin/index.cgi
.Ed
.Pp
To summarize, every virtual host has its own static root resource in its
own subdirectory, but all virtual hosts' static root resources must have
the same name, either the default "index.html" or another name specified by
the -e option.  If a dynamic resource is specified as the root resource,
then it is shared by all virtual hosts, and resides in the (S)CGI
subdirectory, as defined by the -d option.  If a dynamic root resource
needs to know the virtual host for which it is being executed, it must
examine the SERVER_NAME environment variable.
.It -d
The -d option, if present, specifies the subdirectory of the server root
directory which contains CGI and/or SCGI programs.  The value given to this
option cannot contain the directory separator character "/" anywhere in it,
nor can it be ".." or "." by itself.  The directory specified must be one
level below the server root directory.  The directory must be readable and
searchable for the user or group specified with the -u and -g options.  The
directory may contain any subdirectory structure the user wishes to create
in it.  Only executables in the (S)CGI directory, or its subdirectories,
can be run.  Drood refuses to execute symbolic links.
.Pp
All virtual hosts share the (S)CGI directory.  If (S)CGI programs wish to
know what hostname they have been invoked for, they must examine the
SERVER_NAME environment variable.  If no "Host" header was supplied with
the request, then SERVER_NAME will be set to the string "default".  If the
-d option is not present, executables cannot be run.  Requests for dynamic
resources must include the dynamic path in their URIs, because this is how
drood differentiates static requests from dynamic requests.  Eg., if -d is
"cgi-bin", then
.Bd -literal -offset left
http://host.domain/cgi-bin/foobar.cgi
.Ed
.Pp
is a request to run foobar.cgi from the cgi-bin subdirectory of the server
root directory.  If the -d option was not set, then this would be
interpreted as a request for a static resource named foobar.cgi located in
the cgi-bin subdirectory of the host.domain subdirectory of the server root
directory.
.Pp
Note that if you have a (S)CGI directory and you subsequently decide to run
drood without the -d option, that directory will be considered a valid
directory containing static resources for a virtual host, and clients may
request the non-executable resources in the directory, if the client knows
their names, by providing a "Host" header set to the name of the directory.
Drood will always refuse to send files to which it has executable
permission to clients, no matter where they reside, but if you have data
files in your (S)CGI directory, it's a good idea to remove the (S)CGI
directory if you are not running drood with the -d option anymore.  You
should at least change the (S)CGI directory's ownership and/or permissions
to make it inaccessible by drood.
.Pp
(S)CGI programs need only precede their response bodies with a
"Content-Type" header and a blank line.  Drood will chunk response data for
(S)CGI programs invoked over persistent connections.  The (S)CGI program's
response header will be supplemented with a response status line, a "Date"
header if one is missing, a "Server" header, and if the (S)CGI response header
does not include a "Content-Length" header and the connection is
persistent, a "Transfer-Encoding: chunked" header.  If a "Location" header
is provided by the (S)CGI program, then drood with create a "302 Found"
response to send to the client, which will cause the client to redirect
itself to the URI specified in the "Location" header.  Otherwise, drood
will compose a "200 OK" response status line.
.Pp
The stderr of CGI programs is discarded. For testing purposes run drood
with -x option and the CGI stderr will be connected to the terminal from
which drood is running.  For production use, CGI programs must send
diagnostic output to files, or use syslog(3).  The SCGI server determines
how stderr is handled for SCGI programs.
.Pp
The following environment variables are set for (S)CGI programs:
.Pp
REMOTE_ADDR, REQUEST_METHOD, SCRIPT_NAME, QUERY_STRING, HTTP_USER_AGENT,
HTTP_COOKIE, SERVER_SOFTWARE, SERVER_NAME, GATEWAY_INTERFACE, 
CONTENT_TYPE, and CONTENT_LENGTH.
.Pp
Additionally, SCGI is set for SCGI programs, only.
.Pp
Drood cannot recognize virtual paths for (S)CGI programs, so
PATH_TRANSLATED and PATH_INFO are not set.
.Pp
.It -h
.It -s
Including one or both of these options, if and only if the -d option has
also been included on the command-line, causes drood to recognize programs
in the (S)CGI directory with a .scgi filename suffix to be SCGI programs,
and to forward client requests for those programs to a specified SCGI
server.
.Pp
The -h option, if present, must be followed by the hostname or address of
the host where the SCGI server is running, expressed in either IPv4 or IPv6
presentation format.  The -s option specifies the port number the SCGI
server is listening on.  If one of these options is present, but not the
other, then the other option is set to its default value.  The default
value for -h is the IPv4 loopback address, 127.0.0.1, and the default value
for -s is 4000, the default SCGI server port.
.Pp
.It -c
The -c option, if present, makes drood chroot(2) to the server root
directory.  Note that to run CGI programs from a chrooted drood, they must
be statically compiled, as no system shared libraries will be available 
then the CGI programs are run.
.It -p
The -p option specifies the port to listen on.  This defaults to 80 if not
specified.
.It -i
By default, drood accepts connections on all interfaces it can find capable
of IPv4 or IPv6.  The -i option, when present, overrides this behavior, by
limiting drood to accepting connections from a specified interface only.
The option accepts the IP address of the desired interface as an argument.
The address must be expressed in the presentation format for either IPv4 or
IPv6.
.Pp
The intent of this option is to allow the administrator to prevent the
outside world from accessing a server during testing by making drood bind
only to one interface (eg., the loopback address).
.It -u
.It -g
Drood must be started as root in order to bind to port 80, but it tries to
setuid to user "nobody" and setgid to group "nobody" after binding the
listening socket.  If it is unsuccessful in either attempt, the server will
exit.  The -u and the -g options may be used to specify a different user
and group.  If you use the -p option to make drood listen on a port higher
than port 1023, then drood can be started as any user, but it will not be
able to change to user and group "nobody".  In this case, you must provide
the -u and -g options to specify the user and group that drood will begin
running as and stay running as, or the server will exit.
.It -t
When reading a request header from a client, drood drops the connection if
it cannot read the complete request header within twenty seconds.  The -t
option, if present, specifies a different read timeout value, in seconds.
Clients typically keep a connection to a server open to aid responsiveness
if the user subsequently requests more resources from the same server, and
rely on the server to timeout idle connections.  The value must be in the
range 1-60 seconds.  It is important this value take into consideration the
latency of drood on your system under load, and the latency of your network
interfaces, and the possible latency of clients behind firewalls on busy
networks.  If the server starts dropping connections under load, and you
haven't run out of resources, you may have the read timeout set too low.
.It -w
When drood is in the process of writing a requested resource to a client,
the connection will be dropped if 60 minutes has elapsed since it started
writing the resource.  This value may be changed with the -w option, and must
be in range of 0-120 minutes.  Setting this value to 0, disables the write
timeout.  In this case, the client may take as long as it likes to read a
requested resource.  Note that persistent connections may exist for longer
periods than this value specifies, but each individual resource requested
over such a connection must be read by the client within this number of
minutes, or the connection will be dropped.
.It -q
The -q option, if present, specifies the backlog of client connections
queued by the OS kernel for the server to subsequently service.  This value
defaults to 1000.  Note that the OS kernel actually uses a queue of 1.5
times the size of the specified value.  Connections arriving when the queue
is full are dropped by the kernel.  This value should be set to a generous
value.  If the system starts rejecting connections under load, and you're
sure you haven't run out of resources, try increasing the size of the
backlog queue.
.It -x
The -x option, if present, prevents drood from becoming a daemon.  It will
then run in the foreground of the terminal where it was started, and may be
stopped with signals (ie., Control-C).  The server also will not write its
pid to /var/run/drood.pid when the -x option is used.
.It -m
The -m option specifies how many instances of drood should be run, and
defaults to 1.  This option is intended for use on multiple-core or
multiple-processor machines, where running a separate instance of drood for
each core/processor may dramatically increase the number of requests per
second drood can process.
.It -n
The -n option turns off transaction logging.  System errors are always
reported and may not be turned off.  Drood logs transactions and system
errors via syslog(3), which will have a negative impact upon performance.
Turning off transaction logging should make the server faster.
.Pp
A typical transaction log entry is below.  The single line from the access
log has been folded at whitespace to make it easier to read.  Everything up
to the third colon is the prefix added by syslogd.  What follows after is
the data for one client request, consisting of the client IP address in
brackets, the client user-agent string enclosed in braces, the request
method in parenthesis, the request URI delimited by angle brackets, and
finally, the URI submitted in the "Referer" header of the request,
delimited by angle brackets.  If no "Referer" header is present in a
request, the last set of angle brackets will contain the string "no referer
header".
.Pp
Note that the IP address will be expressed in IPv6 presentation format by
default.  Clients making IPv4 connections will have their IP addresses
written as IPv4-mapped IPv6 addresses, as in the example below, unless the
-i option has been specified with an IPv4 address.  In that case the
address with appear in the presentation format for IPv4.
.Pp
Items are separated by single space characters.  The purpose of delimiting
each component with its own delimiter characters, is to simplify the
writing of programs to extract data from log files.
.Bd -literal -offset left
Jun 13 11:30:34 localhost drood[2517]: [::ffff:127.0.0.1]
.br
{Mozilla/5.0 (X11; U; FreeBSD i386; en-US; rv:1.8.1.4)
.br
Gecko/20070609 Firefox/2.0.0.4} (GET) <http://127.0.0.1/turn3.png>
.br
<http://127.0.0.1/index.css>
.Ed
.Pp
Where log message end up is determined by /etc/syslog.conf, most likely in
/var/log/messages.  Add the following three lines to /etc/syslog.conf if
you want drood messages to be written to their own files:
.Bd -literal -offset left
!drood
*.>=notice[4 tab characters]/var/log/drood-error.log
*.<notice[4 tab characters]/var/log/drood-access.log
.Ed
.Pp
Then to get syslogd to start using the logs, do:
.Bd -literal -offset left
touch /var/log/drood-error.log /var/log/drood-access.log
kill -HUP `cat /var/run/syslog.pid`
.Ed
.El
.Sh AUTHORS
.An James Bailie Aq jimmy@mammothcheese.ca
.br
http://www.mammothcheese.ca
